# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimString,
    HprimForeign,
)
from driverng.astm.parser import RecursiveList
import logging
logger = logging.getLogger("driverng.drivers.maldi")


class MaldiSegmentH(HeaderSegment):

    patients = HprimForeign("P", "header")
    orders = HprimForeign("O", "header")
    queries = HprimForeign("Q", "header")
    results = HprimForeign("R", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class MaldiSegmentP(Segment):
    orders = HprimForeign("O", "patients")


class MaldiSegmentO(Segment):
    instrument_id = HprimString((3, 1))
    target_plate_id = HprimString((3, 2))
    project_name = HprimString((3, 3))
    empty_field = HprimString(4, necessary=True)
    results = HprimForeign("R", "orders")


class MaldiSegmentO_RES(Segment):
    accession_number = HprimString((3, 1))
    accession_name = HprimString((3, 2))
    analyte_name = HprimString((3, 3))
    measurement_datetime = HprimString((3, 4))
    instrument_id = HprimString((3, 5))
    target_plate_id = HprimString((3, 6))
    target_plate_position = HprimString((3, 7))
    position = HprimString((3, 8))
    results = HprimForeign("R", "orders")


class MaldiSegmentR(Segment):
    # Envoyé avec la demande
    # ORG: Isolation ORG result
    accession_number = HprimString((3, 1))
    accession_name = HprimString((3, 2))
    analyte_name = HprimString((3, 3))
    id_result = HprimString((3, 4), necessary=True)
    isolation_seq_number = HprimString((3, 5))
    target_plate_id = HprimString((3, 6))
    target_plate_position = HprimString((3, 7))
    organism_code = HprimString((4, 1))
    # A: high
    # B: low
    # C: unknow
    consistency_category = HprimString((4, 3))
    # RUO: research-use-only
    # IVD: in-vitro-diagnostic
    project_type = HprimString((4, 5))
    # STD: standard
    # QC: quality control
    # BC: sepsityper
    # MYC: mycobacteria
    sample_type = HprimString((4, 6))
    validation_flag = HprimString((4, 7))
    target_plate_position = HprimString((7, 1))
    # Optional default 0
    target_plate_chip = HprimString((7, 2), necessary=True)
    empty_field = HprimString(8)
    empty_field2 = HprimString(9, necessary=True)


class MaldiSegmentR_PRO(Segment):
    # PRO: Probability PRO result
    id_result = HprimString((3, 4))
    # Always 1
    isolation_seq_number = HprimString((3, 5))
    isolation_match_number = HprimString((3, 6))
    organism_code = HprimString((4, 1))
    organism_name = HprimString((4, 2))
    # +++: green
    # ++: green
    # +: yellow
    # -: red
    qualitative_probability = HprimString((4, 3))
    # 0 <= score value <= 3.0:w
    quantitative_probability = HprimString((4, 4))
    matched_pattern_uuid = HprimString((4, 5))
    matched_pattern_library = HprimString((4, 6))


class MaldiSegmentQ(Segment):
    instrument_id = HprimString((3, 1))
    target_plate_id = HprimString((3, 2))
    project_name = HprimString((3, 3))


class MaldiSegmentL(Segment):
    terminaison_code = HprimString(3, default="N")


class Driver(AstmMessageV2):
    SPEC = {
        "code": "bruker",
        "label": "Maldi BRUKER"
    }
    SEGMENT_CLASS_MAP = {
        "H": MaldiSegmentH,
        "P": MaldiSegmentP,
        "Q": MaldiSegmentQ,
        "O": MaldiSegmentO,
        "O_RES": MaldiSegmentO_RES,
        "R": MaldiSegmentR,
        "R_PRO": MaldiSegmentR_PRO,
        "L": MaldiSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    def create(self, **kwargs):
        message = MaldiSegmentH(type_segment="H", **kwargs)
        message.end = MaldiSegmentL(type_segment="L")
        return message

    def parse_segment(self, line, separators):
        li = line.split("|")
        segment = super(Driver, self).parse_segment(line, separators)
        if segment.type_segment == 'O':
            if len(li[2]) == 3:
                seg_o = MaldiSegmentO()
            else:
                seg_o = MaldiSegmentO_RES()
            splitted = RecursiveList.split(line, separators)
            seg_o.populate(splitted)
            return seg_o
        if segment.type_segment == 'R':
            if segment.id_result not in ('ORG', 'PRO'):
                logger.info("segment R %d inattendu, ce segment n'est pas traité" % (segment.id_result, ))
            else:
                if segment.id_result == 'ORG':
                    seg_r = MaldiSegmentR()
                else:
                    seg_r = MaldiSegmentR_PRO()
                splitted = RecursiveList.split(line, separators)
                seg_r.populate(splitted)
                return seg_r

        return segment

    def encode_worklist(self, worklist, context):
        message = self.create()
        for sample in worklist.get_bruker_worklist():
            # A vérifier s'il faut enlever le type d'échantillon de la demande
            # Normalement il doit être transmis dans le segment R dans
            # organism_code
            if sample.bruker_type_echantillon == "0":
                echantillon = "STD"  # noqa
            elif sample.bruker_type_echantillon == "1":
                echantillon = "QC"  # noqa
            elif sample.bruker_type_echantillon == "2":
                echantillon = "BC"  # noqa
            elif sample.bruker_type_echantillon == "3":
                echantillon = "MYC"  # noqa
            else:
                echantillon = sample.bruker_type_echantillon    # noqa
            # Echantillon QC
            if sample.bruker_type_echantillon == "1":
                tube_bruker = "BTS"
                accession_name = "0"
            else:
                tube_bruker = "{}-{}".format(
                    sample.bruker_position,
                    sample.sample_id
                )
                accession_name = "1"
            if sample.last_name and sample.first_name:
                nom_patient = "{} {}".format(sample.last_name, sample.first_name)
            else:
                nom_patient = ""
            if sample.bruker_type_echantillon == "1":
                nom_patient = "QC"
            message.results.add(
                accession_number=tube_bruker,
                accession_name=accession_name,
                analyte_name=nom_patient,
                target_plate_position=sample.bruker_position,
            )
        if message.results:
            message.orders.add(
                target_plate_id=sample.bruker_projet,
                project_name=sample.bruker_projet_name
            )
        return message

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                worklist.get_or_create(
                    type="patient",
                    sample_id=seg_q.target_plate_id,
                )
        else:
            worklist = Worklist(context="result")
            for seg_p in message.patients:
                for seg_o in seg_p.orders:
                    indice_confiance = ""
                    for seg_r in seg_o.results:
                        if seg_r.id_result == "ORG":
                            # traitement de l'indice de confiance
                            indice_confiance = "{}".format(seg_r.consistency_category)
                    sample_id = seg_o.accession_number.split("-")
                    # je conserve cela afin de prendre les résultats manuels
                    # dont la plaque n'a pas été importée de MCA et
                    # ceux dont le plan de plaque a été importé de MCA
                    # A conserver pour les tests de qualif et à modifier
                    # ensuite
                    if len(sample_id) == 3:
                        # Résultat avec plan de plaque ancien système
                        sample = worklist.get_or_create(
                            sample_id=sample_id[2],
                            type="patient",
                        )
                    elif len(sample_id) == 2:
                        # Résultat avec plan de plaque MCA
                        sample = worklist.get_or_create(
                            sample_id="{}_{}_{}".format(sample_id[1], seg_o.target_plate_position, indice_confiance),
                            type="patient",
                        )
                    elif sample_id[0] == "BTS":
                        sample = worklist.get_or_create(
                            sample_id="{}_{}_{}".format(sample_id[0], seg_o.target_plate_position, indice_confiance),
                            type="patient",
                        )
                    else:
                        continue
                    date_resu = datetime.datetime.strptime(
                        seg_o.measurement_datetime.split('.')[0],
                        "%Y-%m-%dT%H:%M:%S"
                    )
                    for nb, seg_r in enumerate(seg_o.results, 1):
                        commentaires = []
                        if seg_r.id_result == "ORG":
                            sample.results.add(
                                code="ORG",
                                value=seg_r.organism_code,
                                flags=[seg_r.consistency_category],
                                date=date_resu,
                            )
                        elif seg_r.id_result == "PRO":
                            if seg_o.position:
                                code = "MALDI_{}".format(
                                    seg_o.position,
                                )
                            elif seg_o.target_plate_id:
                                code = "MALDI_{}-{}".format(
                                    seg_o.target_plate_id,
                                    seg_o.target_plate_position,
                                )
                            else:
                                code = "MALDI_{}".format(
                                    seg_o.target_plate_position
                                )
                            if nb == 2:
                                commentaires = ['Premier resultat']
                            # On associe le code du germe avec le libellé du
                            # germe
                            value = "{}_{}".format(
                                seg_r.organism_code,
                                seg_r.organism_name,
                            )
                            sample.results.add(
                                code=code,
                                value=value,
                                flags=[
                                    seg_r.quantitative_probability,
                                ],
                                comments=commentaires,
                                date=date_resu,
                            )
        return worklist
